@@ -4,13 +4,7 @@ from .sdk.datasets import EvaluationDataset, LaminarDataset
 from .sdk.evaluations import evaluate
 from .sdk.laminar import Laminar
 from .sdk.types import SessionRecordingOptions, MaskInputOptions
-from .sdk.types import (
-    AgentOutput,
-    FinalOutputChunkContent,
-    HumanEvaluator,
-    RunAgentResponseChunk,
-    StepChunkContent,
-)
+from .sdk.types import HumanEvaluator
 from .sdk.decorators import observe
 from .sdk.types import LaminarSpanContext
 from .opentelemetry_lib.litellm import LaminarLiteLLMCallback
@@ -20,11 +14,9 @@ from .opentelemetry_lib.tracing.processor import LaminarSpanProcessor
 from .opentelemetry_lib.tracing.tracer import get_laminar_tracer_provider, get_tracer
 
 __all__ = [
-    "AgentOutput",
     "AsyncLaminarClient",
     "Attributes",
     "EvaluationDataset",
-    "FinalOutputChunkContent",
     "HumanEvaluator",
     "Instruments",
     "Laminar",
@@ -33,8 +25,6 @@ __all__ = [
     "LaminarLiteLLMCallback",
     "LaminarSpanContext",
     "LaminarSpanProcessor",
-    "RunAgentResponseChunk",
-    "StepChunkContent",
     "get_laminar_tracer_provider",
     "get_tracer",
     "evaluate",
