@@ -1,9 +1,10 @@
 from contextlib import contextmanager
-from typing import Generator, Tuple
+from typing import Generator, Iterator, Tuple
 
 from opentelemetry import trace
 from opentelemetry.context import Context
 from lmnr.opentelemetry_lib.tracing import TracerWrapper
+from lmnr.opentelemetry_lib.tracing.span import LaminarSpan
 
 
 def get_laminar_tracer_provider() -> trace.TracerProvider:
@@ -14,7 +15,7 @@ def get_laminar_tracer_provider() -> trace.TracerProvider:
 def get_tracer(flush_on_exit: bool = False):
     wrapper = TracerWrapper()
     try:
-        yield wrapper.get_tracer()
+        yield LaminarTracer(wrapper.get_tracer())
     finally:
         if flush_on_exit:
             wrapper.flush()
@@ -27,7 +28,7 @@ def get_tracer_with_context(
     """Get tracer with isolated context. Returns (tracer, context) tuple."""
     wrapper = TracerWrapper()
     try:
-        tracer = wrapper.get_tracer()
+        tracer = LaminarTracer(wrapper.get_tracer())
         context = wrapper.get_isolated_context()
         yield tracer, context
     finally:
@@ -35,13 +36,17 @@ def get_tracer_with_context(
             wrapper.flush()
 
 
-def copy_current_context() -> Context:
-    """Copy the current isolated context for use in threads/tasks."""
-    wrapper = TracerWrapper()
-    return wrapper.get_isolated_context()
+class LaminarTracer(trace.Tracer):
+    _instance: trace.Tracer
 
+    def __init__(self, instance: trace.Tracer):
+        self._instance = instance
 
-def set_context_for_thread(context: Context) -> None:
-    """Set the isolated context for the current thread."""
-    wrapper = TracerWrapper()
-    wrapper.set_isolated_context(context)
+    def start_span(self, *args, **kwargs) -> trace.Span:
+        span = LaminarSpan(self._instance.start_span(*args, **kwargs))
+        return span
+
+    @contextmanager
+    def start_as_current_span(self, *args, **kwargs) -> Iterator[trace.Span]:
+        with self._instance.start_as_current_span(*args, **kwargs) as span:
+            yield LaminarSpan(span)
