@@ -1,5 +1,4 @@
 import asyncio
-import logging
 import orjson
 import os
 import time
@@ -8,12 +7,17 @@ from opentelemetry import trace
 
 from lmnr.sdk.decorators import observe
 from lmnr.sdk.browser.utils import retry_async
+from lmnr.sdk.browser.background_send_events import (
+    get_background_loop,
+    track_async_send,
+)
 from lmnr.sdk.client.asynchronous.async_client import AsyncLaminarClient
 from lmnr.opentelemetry_lib.tracing.context import get_current_context
 from lmnr.opentelemetry_lib.tracing import TracerWrapper
+from lmnr.sdk.log import get_default_logger
 from lmnr.sdk.types import MaskInputOptions
 
-logger = logging.getLogger(__name__)
+logger = get_default_logger(__name__)
 
 OLD_BUFFER_TIMEOUT = 60
 CDP_OPERATION_TIMEOUT_SECONDS = 10
@@ -499,12 +503,14 @@ INJECT_PLACEHOLDER = """
 
         function heartbeat() {
             // Add heartbeat events
-            setInterval(() => {
-                window.lmnrRrweb.record.addCustomEvent('heartbeat', {
-                    title: document.title,
+            setInterval(
+                () => {
+                    window.lmnrRrweb.record.addCustomEvent('heartbeat', {
+                        title: document.title,
                         url: document.URL,
                     })
-                }, HEARTBEAT_INTERVAL
+                },
+                HEARTBEAT_INTERVAL,
             );
         }
 
@@ -785,10 +791,13 @@ async def start_recording_events(
         logger.debug("Failed to inject session recorder, not registering bindings")
         return
 
+    # Get the background loop for async sends (independent of CDP's loop)
+    background_loop = get_background_loop()
+
     # Buffer for reassembling chunks
     chunk_buffers = {}
 
-    async def send_events_from_browser(chunk):
+    async def send_events_from_browser(chunk: dict):
         try:
             # Handle chunked data
             batch_id = chunk["batchId"]
@@ -817,9 +826,13 @@ async def start_recording_events(
                 # Parse the JSON
                 events = orjson.loads(full_data)
 
-                # Send to server
+                # Send to server in background loop (independent of CDP's loop)
                 if events and len(events) > 0:
-                    await client._browser_events.send(lmnr_session_id, trace_id, events)
+                    future = asyncio.run_coroutine_threadsafe(
+                        client._browser_events.send(lmnr_session_id, trace_id, events),
+                        background_loop,
+                    )
+                    track_async_send(future)
 
                 # Clean up buffer
                 del chunk_buffers[batch_id]
@@ -843,7 +856,7 @@ async def start_recording_events(
             return
         if event["executionContextId"] != isolated_context_id:
             return
-        await send_events_from_browser(orjson.loads(event["payload"]))
+        asyncio.create_task(send_events_from_browser(orjson.loads(event["payload"])))
 
     await cdp_client.send.Runtime.addBinding(
         {
