@@ -1,13 +1,6 @@
-from lmnr.sdk.client.asynchronous.resources.agent import AsyncAgent
 from lmnr.sdk.client.asynchronous.resources.browser_events import AsyncBrowserEvents
 from lmnr.sdk.client.asynchronous.resources.evals import AsyncEvals
 from lmnr.sdk.client.asynchronous.resources.tags import AsyncTags
 from lmnr.sdk.client.asynchronous.resources.evaluators import AsyncEvaluators
 
-__all__ = [
-    "AsyncAgent",
-    "AsyncEvals",
-    "AsyncBrowserEvents",
-    "AsyncTags",
-    "AsyncEvaluators"
-]
+__all__ = ["AsyncEvals", "AsyncBrowserEvents", "AsyncTags", "AsyncEvaluators"]
