@@ -1,7 +1,6 @@
-from lmnr.sdk.client.synchronous.resources.agent import Agent
 from lmnr.sdk.client.synchronous.resources.browser_events import BrowserEvents
 from lmnr.sdk.client.synchronous.resources.evals import Evals
 from lmnr.sdk.client.synchronous.resources.tags import Tags
 from lmnr.sdk.client.synchronous.resources.evaluators import Evaluators
 
-__all__ = ["Agent", "Evals", "Evaluators", "BrowserEvents", "Tags"]
\ No newline at end of file
+__all__ = ["Evals", "Evaluators", "BrowserEvents", "Tags"]
