@@ -8,8 +8,10 @@ from lmnr.opentelemetry_lib.tracing.context import (
     CONTEXT_USER_ID_KEY,
     attach_context,
     get_event_attributes_from_context,
+    push_span_context,
 )
 from lmnr.opentelemetry_lib.tracing.instruments import Instruments
+from lmnr.opentelemetry_lib.tracing.processor import LaminarSpanProcessor
 from lmnr.opentelemetry_lib.tracing.tracer import get_tracer_with_context
 from lmnr.opentelemetry_lib.tracing.attributes import (
     ASSOCIATION_PROPERTIES,
@@ -203,6 +205,46 @@ class Laminar:
             force_http=force_http,
         )
 
+        cls._initialize_context_from_env()
+
+    @classmethod
+    def _initialize_context_from_env(cls) -> None:
+        """Attach upstream Laminar context from the environment, if provided."""
+        env_context = os.getenv("LMNR_SPAN_CONTEXT")
+        if not env_context:
+            return
+
+        try:
+            laminar_context = LaminarSpanContext.deserialize(env_context)
+        except Exception as exc:  # pylint: disable=broad-exception-caught
+            cls.__logger.warning(
+                "LMNR_SPAN_CONTEXT is set but could not be deserialized: %s", exc
+            )
+            return
+
+        try:
+            otel_span_context = LaminarSpanContext.try_to_otel_span_context(
+                laminar_context, cls.__logger
+            )
+        except ValueError as exc:
+            cls.__logger.warning(
+                "LMNR_SPAN_CONTEXT is set but invalid span context provided: %s", exc
+            )
+            return
+
+        base_context = trace.set_span_in_context(
+            trace.NonRecordingSpan(otel_span_context), get_current_context()
+        )
+        processor = TracerWrapper.instance._span_processor
+        if isinstance(processor, LaminarSpanProcessor):
+            processor.set_parent_path_info(
+                otel_span_context.span_id,
+                laminar_context.span_path,
+                laminar_context.span_ids_path,
+            )
+        push_span_context(base_context)
+        cls.__logger.debug("Initialized Laminar parent context from LMNR_SPAN_CONTEXT.")
+
     @classmethod
     def is_initialized(cls):
         """Check if Laminar is initialized. A utility to make sure other
